﻿// --------------------------------------------------------------------------------------
//
// LimitedMotor.CS                                                 Ulrich Müller 04.03.09
//                                                                 Aspire C# 2005 Express
// Betrieb eines Motors dessen Bewegungsraum durch zwei Endtaster begrenzt wird
//
// --------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace FishFaceTX {
  public class LimitedMotor : DeviceBase {

    public delegate void Changed(LimitedMotor sender);
    public event Changed LeftToTrue;
    public event Changed RightToTrue;

    private Motor       actMotor;
    private BinaryInput leftSensor;
    private BinaryInput rightSensor;
    private Dir         actState;
    private Dir         oldState;

    public LimitedMotor(FishControl fc, Ctr ctrId, Mot OpMotor, Unv EndLeft, Unv EndRight)
      : this(fc, false, ctrId, OpMotor, EndLeft, EndRight) {
    }
    public LimitedMotor(FishControl fc, bool WithEvents, Ctr ctrId, Mot OpMotor,
      Unv EndLeft, Unv EndRight)
      : base(fc, false, (int)ctrId, 105) {
      this.actMotor    = new Motor(fc, ctrId, OpMotor);
      this.leftSensor  = new D5KInput(fc, ctrId, EndLeft);
      this.rightSensor = new D5KInput(fc, ctrId, EndRight);
      this.actState    = Dir.Off;
      base.deviceType  = "LimitedMotor";
      base.withEvents  = WithEvents;
    }

    protected override void EventLoop() {
      oldState = Dir.Off;
      while (!tx.NotHalt && !tx.IsESC) {
        actState = ActualState;
        if (actState != oldState) {
          if (actState == Dir.Left) {
            if (LeftToTrue != null) LeftToTrue(this);
          }
          else if (actState == Dir.Right) {
            if (RightToTrue != null) RightToTrue(this);
          }
          oldState = actState;
        }
        Thread.Sleep(base.threadInterval);
      }
    }

    public override string ConnectorName {
      get {
        return base.deviceType;
      }
    }
    public int MotorNumber {
      get {
        return actMotor.ConnectorNumber;
      }
    }
    public string MotorName {
      get {
        return actMotor.ConnectorName;
      }
    }
    public int ConNumberLeft {
      get {
        return leftSensor.ConnectorNumber;
      }
    }
    public string ConNameLeft {
      get {
        return leftSensor.ConnectorName;
      }
    }
    public int ConNumberRight {
      get {
        return rightSensor.ConnectorNumber;
      }
    }
    public string ConNameRight {
      get {
        return rightSensor.ConnectorName;
      }
    }
    public bool IsLeft {
      get {
        return (actState == Dir.Left);
      }
    }
    public bool IsRight {
      get {
        return (actState == Dir.Right);
      }
    }
    public Dir ActualState {
      get {
        Dir act;
        if (actMotor.State == Dir.On) act = Dir.On;
        else if (leftSensor.IsTrue) act = Dir.Left;
        else if (rightSensor.IsTrue) act = Dir.Right;
        else act = Dir.Off;
        return act;
      }
    }

    public void StartLeft(int speed) {
      actState = Dir.On;
      actMotor.Go(Dir.Left, speed);
    }
    public void StartLeft() {
      actState = Dir.On;
      actMotor.Left();
    }
    public void MoveLeft(int speed) {
      StartLeft(speed);
      WaitForDone();
    }
    public void MoveLeft() {
      StartLeft();
      WaitForDone();
    }
    public void StartRight(int speed) {
      actState = Dir.On;
      actMotor.Go(Dir.Right, speed);
    }
    public void StartRight() {
      actState = Dir.On;
      actMotor.Right();
    }
    public void MoveRight(int speed) {
      StartRight(speed);
      WaitForDone();
    }
    public void MoveRight() {
      StartRight();
      WaitForDone();
    }
    public void WaitForDone() {
      if (actMotor.State == Dir.Left) {
        leftSensor.WaitForTrue();
        actState = Dir.Left;
      }
      else {
        rightSensor.WaitForTrue();
        actState = Dir.Right;
      }
      actMotor.Off();
    }
  }
}
