﻿// --------------------------------------------------------------------------------------
//
// RobMotors.CS                                                    Ulrich Müller 02.11.09
//                                                                 Aspire C# 2005 Express
// Betrieb mehrerer RobMotor im Verbund
//
// --------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace FishFaceTX {
  public struct RobData {
    public RobData(Mot MotNr, int MaxPos) {
      this.MotName = MotNr;
      this.MaxPos = MaxPos;
    }
    public Mot MotName;
    public int MaxPos;
  }

  public class RobMotors : DeviceBase {

    public delegate void PositionS(RobMotors sender, int[] Pos);
    public event         PositionS ChangedPosition;

    private RobMotor[] RMotListe;
    private int[]      PosList;
    private int        anzMot;
    private int        anzPos;
    private Dir        stateValue;
    private bool       noEvent;

    public RobMotors(FishControl fc, Ctr ctrId, params RobData[] RMotList)
      : this(fc, true, ctrId, RMotList) {
    }
    public RobMotors(FishControl fc, bool WithEvents, Ctr ctrId, 
      params RobData[] RMotList) : base(fc, true, (int)ctrId, 102) {
      base.deviceType = "RobMotors";
      this.anzMot     = RMotList.Length;
      this.anzPos     = 0;
      this.stateValue = Dir.Off;
      this.RMotListe  = new RobMotor[anzMot];
      for (int i = 0; i < this.anzMot; i++) {
        this.RMotListe[i] = new RobMotor(fc, true, ctrId,
                                RMotList[i].MotName, RMotList[i].MaxPos);
        if (WithEvents) {
          this.RMotListe[i].WithEvents = true;
          this.RMotListe[i].ChangedPosition += new RobMotor.Position(EventTrigger);
        }
      }
      PosList = new int[anzMot];
      noEvent = true;
    }

    protected override void EventLoop() {
    }
    private void EventTrigger(RobMotor sender, int Pos) {
      if (noEvent == true) return;
      for (int i = 0; i < anzMot; i++) {
        PosList[i] = RMotListe[i].ActualPosition;
      }
      if (ChangedPosition != null) ChangedPosition(this, PosList);
    }

    public override string ConnectorName {
      get {
        return base.deviceType;
      }
    }
    public Dir State {
      get {
        return stateValue;
      }
    }
    public override int ThreadInterval {
      get {
        return base.ThreadInterval;
      }
      set {
        base.ThreadInterval = value;
        for (int i = 0; i < this.anzMot; i++) this.RMotListe[i].ThreadInterval = value;
      }
    }
    public void StartHome() {
      stateValue = Dir.Left;
      noEvent = true;
      for (int i = 0; i < anzMot; i++) RMotListe[i].StartHome();
    }
    public void MoveHome() {
      stateValue = Dir.Left;
      noEvent = true;
      for (int i = 0; i < anzMot; i++) RMotListe[i].DriveHome();
    }
    public void WaitForDone() {
      for (int i = 0; i < anzMot; i++) RMotListe[i].WaitForDone();
      stateValue = Dir.Off;
      noEvent = false;
    }
    public void StartTo(params int[] DestPos) {
      anzPos = DestPos.Length;
      stateValue = Dir.On;
      noEvent = false;
      for (int i = 0; i < anzPos; i++) RMotListe[i].StartTo(DestPos[i]);
    }
    public void MoveTo(params int[] DestPos) {
      StartTo(DestPos);
      WaitForDone();
    }
    public void StartDelta(params int[] DestDelta) {
      anzPos = DestDelta.Length;
      stateValue = Dir.On;
      noEvent = false;
      for (int i = 0; i < anzPos; i++) RMotListe[i].StartDelta(DestDelta[i]);
    }
    public void MoveDelta(params int[] DestDelta) {
      StartDelta(DestDelta);
      WaitForDone();
    }

    public RobMotor this[int index] {
      get {
        return RMotListe[index];
      }
    }
  }
}
