﻿// --------------------------------------------------------------------------------------
//
// RechenTest                                                      Ulrich Müller 20.11.09
//                                                                 Aspire C# 2005 Express
// Kleine Rechenaufgaben, Kontrolle der Lösung über Taschenrechner
//
// Nutzung FishFaceTX.DLL Klasse FishFace
//
// Belegung Interface :
// Y-Achse : M1(EncoderMotor), I1(Endtaster), C1(Impuls)
// X-Achse : M2(EncoderMotor), I2(Endtaster), C2(Impuls)
// Taster  : M3(XS-Motor), I3(Endtaster)
//
// --------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using FishFaceTX;

namespace RechenTest {
  public partial class frmMain : Form {
    FishFace tx = new FishFace();
    Mot xMotor  = Mot.M1;
    Mot yMotor  = Mot.M2;
    Mot taster  = Mot.M3;
    Unv tEnd    = Unv.I3;

    const int txFull = 512;
    const int xMax   = 1040;
    int       xStart = 0;
    int       xDest  = 0;
    const int yMax   = 1040;
    int       yStart = 0;
    int       yDest  = 0;

    private int[,] TasPos ={{940, 380},     // --- Taste 0
                            {820, 380},     // --- Taste 1
                            {820, 600},     // --- Taste 2
                            {820, 780},     // --- Taste 3
                            {660, 380},     // --- Taste 4
                            {660, 600},     // --- Taste 5
                            {660, 780},     // --- Taste 6
                            {530, 380},     // --- Taste 7
                            {530, 600},     // --- Taste 8
                            {530, 780},     // --- Taste 9
                            {940, 1020},    // --- Plus
                            {940, 780},     // --- Gleich
                            {940, 200}      // --- Clear
                    };

    public frmMain() {
      InitializeComponent();
    }

    private void TasterGo() {
      // --- Taste drücken ---
      tx.SetMotor(taster, Dir.Right);
      tx.WaitForInput(tEnd);
      tx.SetMotor(taster, Dir.Left);
      tx.Pause(555);
      tx.SetMotor(taster, Dir.Off);
    }
    private void MoveHome() {
      xDest = 0;
      yDest = 0;
      tx.StartRobMotor(xMotor, Dir.Left, txFull, 9999);
      tx.StartRobMotor(yMotor, Dir.Left, txFull, 9999);
      tx.WaitForMotors(xMotor, yMotor);
      xStart = 0;
      yStart = 0;
    }
    private void MoveTo(int x, int y) {
      int diff, vorz = 1;
      xDest = x > xMax ? xMax : x;
      if (xDest > xStart) {
        tx.StartMotor(xMotor, Dir.Right, txFull, xDest - xStart);
        vorz = 1;
      }
      else if (xStart - xDest > 0) {
        tx.StartRobMotor(xMotor, Dir.Left, txFull, xStart - xDest);
        vorz = -1;
      }

      yDest = y > yMax ? yMax : y;
      if (yDest > yStart) {
        tx.StartMotor(yMotor, Dir.Right, txFull, yDest - yStart);
        vorz = 1;
      }
      else if (yStart - yDest > 0) {
        tx.StartRobMotor(yMotor, Dir.Left, txFull, yStart - yDest);
        vorz = -1;
      }
      diff   = tx.WaitForMotor(xMotor);
      xStart = xDest + diff * vorz;
      diff   = tx.WaitForMotor(yMotor);
      yStart = yDest - diff * vorz;
    }
    
    private void cmdAction_Click(object sender, EventArgs e) {
      int OP1, OP2;
      int i, ptrTas;
      int[] Tasten = new int[24];

      try {
        if (cmdAction.Text == "START") {
          // --- Auf StartPosition fahren ---
          tx.OpenController(txtCom.Text);
          cmdEnde.Text   = "HALT";
          lblStatus.Text = "Fährt auf Home-Position";
          TasterGo();
          MoveHome();
          MoveTo(TasPos[12, 0], TasPos[12, 1]);
          TasterGo();

          lblStatus.Text  = "Gestartet";
          cmdEnde.Text    = "ENDE";
          cmdAction.Text  = "LÖSUNG";
          cmdEnde.Focus();
        }
        else if (cmdAction.Text == "LÖSUNG") {
          // --- Lösung suchen ----
          OP1 = Convert.ToInt32(txtOP1.Text);
          OP2 = Convert.ToInt32(txtOP2.Text);
          
          ptrTas = 0; Tasten[ptrTas++] = 12;  // --- Clear

          // --- Ziffernpositionen für ersten Operanden
          for (i = 0; i < txtOP1.Text.Length; i++)
            Tasten[ptrTas++] = Convert.ToInt32(txtOP1.Text.Substring(i, 1));
          
          Tasten[ptrTas++] = 10;              // --- Addition

          // --- Ziffernpositionen für zweiten Operanden
          for (i = 0; i < txtOP2.Text.Length; i++)
            Tasten[ptrTas++] = Convert.ToInt32(txtOP2.Text.Substring(i, 1));
          
          Tasten[ptrTas++] = 11;              // --- Taste =
          
          // --- Anfahren der erforderlichen Tasten ---
          cmdEnde.Text = "HALT"; lblStatus.Text = "Kontrolle mit dem Taschenrechner";
          for (i = 0; i < ptrTas; i++) {
            MoveTo(TasPos[Tasten[i], 0], TasPos[Tasten[i], 1]);
            TasterGo();
          }
          // --- Anzeige korrektes Ergebnis ---
          txtErgebnis.Text = Convert.ToString(OP1 + OP2);
          lblStatus.Text = "OK, das war's";
          cmdEnde.Text = "ENDE";
        }
      }
      catch (FishFaceException txe) {
        MessageBox.Show(txe.Message, this.Text,
          MessageBoxButtons.OK, MessageBoxIcon.Stop);
      }
      catch (Exception ee) {
        lblStatus.Text = ee.Message;
      }
    }

    private void cmdEnde_Click(object sender, EventArgs e) {
      if (cmdEnde.Text == "HALT") tx.NotHalt = true;
      else {
        tx.CloseController();
        this.Close();
      }
    }

    private void formClosing(object sender, FormClosingEventArgs e) {
      if (cmdEnde.Text == "HALT") e.Cancel = true;
    }
  }
}